@@ -2,17 +2,15 @@
 # housekeeping
 ########################################################################
 
-package LinkedList::Single v1.3.0;
-use v5.24;
+package LinkedList::Single;
 
+use v5.16;
+
+use Carp;
+
+use Scalar::Util    qw( blessed refaddr reftype looks_like_number );
 use Symbol;
 
-use List::Util;
-
-use Carp            qw( carp croak confess                          );
-use Scalar::Util    qw( blessed refaddr reftype looks_like_number   );
-use Storable        qw( dclone                                      );
-
 use overload
 (
     q{bool} =>
@@ -24,39 +22,32 @@ use overload
         #
         # this allows for:
         #
-        # $listh->head; while( $listh ){ ... $listh->next }
+        # $listh->head; while( $listh ){ ... }
 
         my $listh   = shift;
 
-        $$listh && $$listh->[0]
+        $$listh # && @{ $$listh }
     },
 
+    # return a node at the given offset.
+    # update the list to that point.
+
     q{+} =>
     sub
     {
-        # return a node at the given offset.
-        # update the list to that point.
-
-        my ( $listh, $offset )
-        = $_[2]
-        ? @_[1,0]
+        my ( $listh, $offset ) 
+        = $_[2] 
+        ? @_[1,0] 
         : @_[0,1]
         ;
 
         my $node    = $$listh || $listh->head_node;
 
         # i.e., $offset == 0, gets ( 1 .. 0 ) for no change.
-        #
-        # note: this might want to return undef if that node
-        # does not exist -- i.e., more like perly array result.
-        #
-        # note the side effect on += if that change is made:
-        # should $listh be updated to undef or left as-is?
 
         for ( 1 .. $offset )
         {
-            $node
-            or return undef;
+            $node or last;
 
             $node       = $node->[0];
         }
@@ -67,9 +58,9 @@ use overload
     q{+=} =>
     sub
     {
-        my ( $listh, $offset )
-        = $_[2]
-        ? @_[1,0]
+        my ( $listh, $offset ) 
+        = $_[2] 
+        ? @_[1,0] 
         : @_[0,1]
         ;
 
@@ -78,124 +69,35 @@ use overload
         $listh
     },
 
-    q{""} =>
-    sub
-    {
-        my ( $listh )
-        = $_[2]
-        ? $_[1]
-        : $_[0]
-        ;
-
-        $listh->id_string
-    },
 );
 
 ########################################################################
 # package variables
 ########################################################################
 
-our @CARP_NOT   = ( __PACKAGE__ );
+our $VERSION = '1.02';
+$VERSION = eval $VERSION;
 
 # inside-out data for the heads of the lists.
 
 my %rootz   = ();
 
-my $verbose = $ENV{ VERBOSE } // '';
-
 ########################################################################
 # utility subs
 ########################################################################
 
-my $get_node_data
+########################################################################
+# perl's recursive cleanups is no longer an issue after 5.16.
+
+my $cleanup
 = sub
 {
-    my $node    = shift
-    or croak 'Bogus get_node_data: false node';
+    my $node    = shift;
+    @$node      = ();
 
-    my ( undef, @data ) = @$node;
-
-    wantarray
-    ?  @data
-    : \@data
-};
-
-my $set_node_data
-= sub
-{
-    my $node    = shift
-    or croak 'Bogus set_node_data: false node';
-
-    @_  or carp "Oddity: setting empty node data"
-    if $verbose;
-
-    # return the previous data for posterity
-
-    wantarray
-    ?   splice @$node, 1, $#$node, @_
-    : [ splice @$node, 1, $#$node, @_ ]
-};
-
-my $list_next
-= sub
-{
-    my $listh   = shift;
-
-    my $node    = $$listh
-    or croak "Bogus list_next: list handler does not reference a node";
-
-    # return handles assignment in list context
-    # more gracefully by not passing back scalar
-    # undef, remains false in a list context.
-
-    $node->[0]
-    or
     return
 };
 
-########################################################################
-# extract the handler, tail & count args for map, reduce, etc.
-# @CARP_NOT prevents these from reporting errors as being from the
-# calling subs.
-
-my $handler_args
-= sub
-{
-    my ( $listh, $handler, %argz ) = @_;
-
-    blessed $listh
-    or croak "Bogus list handler: un-blessed '$listh'";
-
-    # handler can safely be false, avoid undefs.
-
-    $handler //= '';
-
-    if( exists $argz{ count } )
-    {
-        my $count   = $argz{ count };
-
-        if( looks_like_number $count )
-        {
-            $count < 0
-            and croak "Botched count: negative value '$count'";
-
-            # zero count will never produce any result.
-
-            $count
-            or return;
-        }
-        elsif( $count )
-        {
-            croak "Botched count: non-numeric value '$count'";
-        }
-
-        ( $listh, $handler, $count )
-    }
-    else
-    {
-        ( $listh, $handler )
-    }
-};
 
 ########################################################################
 # public interface
@@ -212,40 +114,40 @@ sub construct
 {
     my $proto   = shift;
 
-    # i.e., a reference to the root node referencing the head node,
-    # which begins life as the tail node and is thus empty.
+    my $listh   = bless \[], blessed $proto || $proto;
 
-    my $listh   = bless \( [ [] ] ), ref $proto || $proto;
-
-    $rootz{ refaddr $listh } = $$listh;
+    $rootz{ refaddr $listh } = [ $$listh ];
 
     $listh
 }
 
 sub initialize
 {
-    # this may not always be called from new(), need
-    # perform some add'l housekeeping.
-    #
-    # this cleans up any existing by de-referencing the
-    # head node, leaves the list at the head (vs. root).
-    #
-    # this leaves $listh referencing the head node, which is
-    # what people usually want.
-
     my $listh   = shift;
-    my $node    = $listh->root_node;
-    ( $node )   = @$node = ( [] );
-    $$listh     = $node;
+
+    # data for the list is on the stack.
+    # otherwise the caller gets back an
+    # empty list.
 
     if( @_ )
     {
-        # start a at the new head == tail node, expand it
-        # into a new tail node + data.
+        # no telling if somone overloaded new and
+        # moved the node. only fix is to re-set the
+        # thing to the head before adding the data.
+        #
+        # or... they know enough to leave it alone
+        # if they want to so this should just take
+        # the location as-is.
+
+        my $node    = $listh->head_node;
 
         ( $node ) = @$node = ( [], $_ )
         for @_;
     }
+    else
+    {
+        $listh->head;
+    }
 
     return
 }
@@ -261,48 +163,22 @@ sub new
 
 sub clone
 {
-    # clone a node with the same head as the current list.
+    # recycle the head node
 
     my $listh   = shift;
-    my $clone   = $listh->new;
 
-    $$clone     = $$listh;
+    my $clone   = bless \[ $$listh ], blessed $listh || $listh;
 
-    $rootz{ refaddr $clone }
-    = $rootz{ refaddr $listh };
-
-    $clone
-}
-
-sub clone_list
-{
-    # clone the contents of $listh into a new list.
-    #
-    # reacall that the linked lists looks like a nested-
-    # struct-from-hell to Perl, at which point dclone will
-    # happily & quickly create a new copy of it all.
-    #
-    # note that assigning $rootz{ r/a $new } will overwrite 
-    # the root, discarding any existing linked list manged
-    # by $clone.
-
-    my $listh   = shift;
-    my $clone   = $listh->new;
-
-    $clone->root_node->[0] = dclone $listh->root_node->[0];
+    $rootz{ refaddr $clone } = $rootz{ refaddr $listh };
 
     $clone
 }
 
 sub DESTROY
 {
-    # note that this could be a clone whose 
-    # root entry was alrady zeroed by the 
-    # main list handler.
+    my $head    = delete $rootz{ refaddr shift };
 
-    my $listh   = shift;
-
-    delete $rootz{ refaddr $listh };
+    $#$head     = -1;
 
     return
 }
@@ -319,6 +195,8 @@ sub truncate
 
     my $next    = splice @$node, 0, 1, [];
 
+    $cleanup->( $next );
+
     $listh
 }
 
@@ -340,22 +218,6 @@ sub replace
 # sets the node. this allows for tell/reset-style stacking of node
 # positions.
 
-sub curr_node
-{
-    # yes, this is a metadata leak.
-    # doing it this way makes it simpler to
-    # deal with things like map and friends
-    # using methods.
-
-    my $listh   = shift;
-    my $node    = $$listh
-    or return;
-
-    wantarray
-    ? @$node
-    :  $node
-}
-
 sub node
 {
     my $listh   = shift;
@@ -366,16 +228,17 @@ sub node
 }
 
 ########################################################################
-# hide extra data in the root node after the first-node ref.
+# hide extra data in the head node after the first-node ref.
 #
 # splice with $#$head works since 1 .. end == length - 1 == offset.
 
 sub set_meta
 {
     my $listh   = shift;
-    my $root    = $listh->root_node;
 
-    @$root      = ( $root->[0], @_ );
+    my $root    = $listh->root;
+
+    splice @$root, 1, $#$root, @_;
 
     $listh
 }
@@ -383,7 +246,8 @@ sub set_meta
 sub add_meta
 {
     my $listh   = shift;
-    my $root    = $listh->root_node;
+
+    my $root    = $listh->root;
 
     push @$root, @_;
 
@@ -392,22 +256,13 @@ sub add_meta
 
 sub get_meta
 {
-    my $listh   = shift;
-    my $root    = $listh->root_node;
+    my $root    = $_[0]->root;
 
     wantarray
     ?   @{ $root }[ 1 .. $#$root ]
     : [ @{ $root }[ 1 .. $#$root ] ]
 }
 
-sub id_string
-{
-    my $listh   = shift;
-    my $root    = $listh->root_node;
-
-    $root->[1] // join '=' => blessed $listh, refaddr $listh
-}
-
 ########################################################################
 # node/list status
 
@@ -415,7 +270,7 @@ sub has_nodes
 {
     # i.e., is the list populated?
 
-    my $root    = $_[0]->root_node;
+    my $root    = $_[0]->root;
 
     !! @{ $root->[0] }
 }
@@ -423,26 +278,10 @@ sub has_nodes
 sub has_next
 {
     # i.e., while( $node->has_next ){ ... }
-    # true if the next node is not the empty sentinel.
 
     my $listh   = shift;
 
-    !! $listh->$list_next
-}
-
-sub has_data
-{
-    # i.e., if( $listh->has_data ) { process the node's data }
-    # note that an "empty" node might have a next on and just
-    # no data at that node.
-    #
-    # a node with only a pointer will have the next-node
-    # reference; the tail node has nothing in it.
-
-    my $listh   = shift;
-    my $node    = $$listh;
-
-    @$node > 1
+    scalar @{ $$listh->[0] }
 }
 
 sub is_empty
@@ -451,55 +290,69 @@ sub is_empty
     # A: it will if there is more than one element.
 
     my $listh   = shift;
-    my $node    = $$listh;
 
-    @$node < 2
-}
-
-sub clear_node_data
-{
-    my $listh   = shift;
-    my $node    = $$listh;
-
-    $#$node     = 0;
-
-    $listh
+    ! @{ $$listh }
 }
 
 sub node_data
 {
     my $listh   = shift;
+    my $node    = $$listh;
 
-    # any data to replace the current data is
-    # left on the stack.
+    # return the existing data, sans the next ref.
 
-    my $node    = $$listh
-    or croak "Bogus node_data: listh handler '$listh' does not reference a node";
-
-    @_
-    ? $node->$set_node_data( @_ )
-    : $node->$get_node_data
+    wantarray
+    ?   @{ $node }[ 1 .. $#$node ]
+    : [ @{ $node }[ 1 .. $#$node ] ]
 }
 
 sub next_data
 {
     my $listh   = shift;
-    my $node    = $$listh
-    or croak "Bogus next_data: listh handler '$listh' does not reference a node";
+    my $node    = $$listh;
+    my $next    = $node->[0];
 
-    local $$listh   = $node->[0];
+    my @valz    = @{ $next }[ 1 .. $#$next ];
 
-    $listh->node_data( @_ )
+    wantarray
+    ?  @valz
+    : \@valz
+}
+
+sub set_data
+{
+    my $listh   = shift;
+    my $node    = $$listh;
+
+    # any data to replace the current data is
+    # left on the stack.
+
+    # return the existing data.
+
+    if( defined wantarray )
+    {
+        my @valz    = @{ $node }[ 1 .. $#$node ];
+
+        @_ and splice @$node, 1, $#$node, @_;
+
+        wantarray
+        ?  @valz
+        : \@valz
+    }
+    else
+    {
+        splice @$node, 1, $#$node, @_;
+
+        return
+    }
 }
 
 sub clear_data
 {
     my $listh   = shift;
+    my $node    = $$listh;
 
-    my $node    = $$listh
-    or return;
-
-    @$node  = ( $node->[0] );
+    splice @$node, 1;
 
     $listh
 }
@@ -530,7 +383,7 @@ sub list_data
 # list (i.e., when OO calls are too expensive).
 #
 # new_head is surgery: replace the head node.
-# leaves most sanity checks in the
+# leaves most sanity checks in the 
 # caller's hands.
 #
 # mainly useful for cross-linked lists.
@@ -554,6 +407,8 @@ sub new_root
 
     my $key     = refaddr $listh;
 
+    $cleanup->( delete $rootz{ $key } );
+
     $rootz{ $key }  = $root;
 
     $listh
@@ -567,17 +422,14 @@ sub new_head
 
     $head       = splice @$root, 0, 1, $head;
 
+    $cleanup->( $head );
+
     $listh->head
 }
 
 sub root_node
 {
-    my $listh   = shift;
-    my $root    = $rootz{ refaddr $listh };
-
-    wantarray
-    ? @{ $root }
-    :    $root
+    scalar $rootz{ refaddr $_[0] }
 }
 
 sub head_node
@@ -586,15 +438,14 @@ sub head_node
 
     my $root    = $listh->root_node;
 
-    wantarray
-    ? @{ $root->[0] }
-    :    $root->[0]
+    $root->[0]
 }
 
 sub root
 {
     my $listh   = shift;
-    $$listh     = $listh->root_node;
+    my $root    = $listh->root_node;
+    $$listh     = $root;
 
     $listh
 }
@@ -602,7 +453,8 @@ sub root
 sub head
 {
     my $listh   = shift;
-    $$listh     = $listh->head_node;
+    my $head    = $listh->head_node;
+    $$listh     = $head;
 
     $listh
 }
@@ -613,387 +465,46 @@ sub head
 sub next
 {
     my $listh   = shift;
-    my $node    = $$listh
-    or croak "Bogus next: list handler does not reference a node";
+    my $node    = $$listh;
 
-    if( @_ )
+    my $count   = @_ ? shift : 1;
+
+    looks_like_number $count
+    or croak "Bogus move: '$count' is not a number";
+
+    for( 1 .. $count )
     {
-        my $offset  = shift;
-
-        looks_like_number $offset
-        or croak "Bogus next: '$offset' is not a number";
-
-        if( $offset > 0 )
-        {
-            # testing @$node avoids walking off the
-            # end of the list, leaves $listh at the
-            # tail rather than undef.
-            #
-            # this also means that while( $listh->next )
-            # can lead to an infinite loop w/o test for
-            # has_next or is_tail or has_data.
-
-            while( @$node && $offset-- )
-            {
-                $node   = $node->[0];
-            }
-
-            $$listh     = $node;
-        }
-        elsif( $offset )
-        {
-            croak "Bogus next: negative offset '$offset'";
-        }
-        else
-        {
-            # offset zero, do nothing.
-        }
+        @$node or last;
+        
+        $node = $node->[0];
     }
-    else
-    {
-        # skip all of the extra work for the
-        # default case of advancing one node.
-
-        $$listh = $node->[0]
-    }
-
-    # note that $listh may reference the tail at
-    # this point but will not be false.
-
-    $listh
-}
-
-sub each
-{
-    my $listh   = shift;
-    my $node    = $$listh
-    or return;
-
-    # leaves $listh parked at the tail rather than
-    # walking off the list.
-
-    @$node
-    or return;
-
-    ( $$listh, my @valz )  = @$node;
-
-    if( wantarray )
-    {
-        @valz
-    }
-    else
-    {
-        \@valz
-    }
-}
-
-########################################################################
-# Notes:
-#
-# * depending on location w/ tail or list having any data
-#   to begin with these may produce an empty list.
-#
-# * result lists begin at the root (vs. head) node since add puts the
-#   next node after the current one.
-
-my $first
-= sub
-{
-    my ( $node, $select ) = @_;
-
-    for(;;)
-    {
-        # only ways out are end of list (undef), end of
-        # count (false), or locating a value (true).
-
-        my ( $next, @buffer ) = @$node
-        or return undef;
-
-        $select->( @buffer )
-        and return $node;
-
-        $node   = $next;
-    }
-};
-
-sub first
-{
-    state $default  
-    = sub
-    {
-        # i.e., find the first node that 
-        # contains a true data value.
-
-        $_ and return $_ for @_;
-        return
-    };
-
-    my ( $listh, $select ) = &$handler_args
-    or return;
-
-    my $node    = $$listh
-    or return;
-
-    @$node
-    or return;
-
-    $node       = $first->( $node->[0], $select || $default )
-    or return;
 
     $$listh     = $node;
 
     $listh
 }
 
-sub map
+# this returns false for the scalar
+# case, where an empty node returns
+# an empty arrayref.
+#
+# Q: Better to reset or stall and EOL?
+# A: Better leaving the reset to the caller, who knows
+#    what to expect in that case.
+
+sub each
 {
-    state $default
-    = sub
-    {
-        my $buffer  = shift;
-
-        @$buffer
-    };
-
-    my ( $listh, $xform, $count ) = &$handler_args;
-
-    $xform  ||= $default;
-
-    # start at the root, not the head since $result->add puts
-    # the data into a new node after the current one.
-
-    my $result  = $listh->new->root;
-    my $node    = $$listh;
-    my @buffer  = ();
-
-    while( ( $node, @buffer ) = @$node )
-    {
-        $result->add( $xform->( \@buffer ) )->next;
-
-        --$count
-        or last;
-    }
-
-    $result->head
-}
-
-sub copy
-{
-    # i.e., map with default xform
-
     my $listh   = shift;
-
-    $listh->map( '' => @_ )
-}
-
-my $node_nonempty
-= sub
-{
-    # i.e., true if the node has any data.
-
-    !! @_
-};
-
-sub prune
-{
-    my ( $listh, $select, $count ) = &$handler_args;
-
-    my $node    = $listh->root_node;
-
-    while( my $next = $node->[0] )
-    {
-        $node->[0] 
-        = $first->
-        (
-            $next,
-            $select || $node_nonempty
-        )
-        or do
-        {
-            $node->[0]  = [];
-            last;
-        };
-
-        --$count
-        or last;
-    }
-    continue
-    {
-        $node   = $node->[0];
-    }
-
-    $$listh = $node;
-
-    $listh
-}
-
-sub grep
-{
-    my ( $listh, $select, $count ) = &$handler_args;
-
-    $count      //= 0;
-
-    my $node    = $$listh;
-    my $result  = $listh->new->root;
-    my @buffer  = ();
-
-    $select     ||= $node_nonempty;
-
-    while( ( $node, @buffer ) = @$node )
-    {
-        $select->( @buffer )
-        and
-        $result->add( @buffer )->next;
-
-        --$count
-        or last;
-    }
-
-    # in a void context replace the existing list in place.
-
-    defined wantarray
-    ? $result->head
-    : $result->head->clone( $listh )
-}
-
-sub reduce
-{
-    my ( $listh, $handler, $count ) = &$handler_args;
-
-    $handler
-    or croak "Bogus reduce: false reduce operator '$handler'";
-
-    my $result  = undef;
-    my @buffer  = ();
-
     my $node    = $$listh;
 
-    while( ( $node, @buffer ) = @$node )
-    {
-        $result = $handler->( $result, @buffer );
+    $node
+    or return;
 
-        --$count
-        or last;
-    }
+    ( $$listh, my @valz )  = @$node;
 
-    $result
-}
-
-sub sum
-{
-    state $handler
-    = sub
-    {
-        List::Util::sum @_
-    };
-
-    my $listh   = shift;
-
-    $listh->reduce( $handler, @_ )
-}
-
-sub sort
-{
-    state $default
-    = sub
-    {
-        my ( $curr, $next ) = ( $_[0]->[0], $_[1]->[0] );
-
-        $curr <=> $next
-        or
-        $curr cmp $next
-    };
-
-    my ( $listh, $compare, $count ) = &$handler_args;
-
-    $compare    ||= $default;
-    $count      //= 0;
-
-    my $sorted  = $listh->new;
-
-    my $src     = $listh->head_node;
-    my $root    = $sorted->root_node;
-    my $dst     = '';
-    my $next    = '';
-    my @src_d   = ();
-    my @dst_d   = ();
-
-    while( ( $src, @src_d ) = @$src )
-    {
-        $dst    = $root;
-
-        # walk down until the first point at which the
-        # next node is > this one & insert this one 
-        # prior to it.
-
-        while
-        (
-            $next   = $dst->[0]
-            and
-            ( undef, @dst_d ) = @$next
-        )
-        {
-            0 > $compare->( \@src_d, \@dst_d )
-            and last;
-
-            $dst    = $next;
-        }
-
-        $dst->[0]   = [ $next, @src_d ];
-    }
-    continue
-    {
-        --$count
-        or last;
-    }
-
-    $sorted->head
-}
-
-sub num_sort
-{
-    state $compare
-    = sub
-    {
-        my $a       = shift
-        or return;
-
-        my $b       = shift
-        or return;
-
-        List::Util::first
-        {
-            $a->[$_] <=> $b->[$_]
-        }
-        ( 0 .. $#$a )
-    };
-
-    my $listh   = shift;
-
-    $listh->sort( $compare )
-}
-
-sub lex_sort
-{
-    state $compare
-    = sub
-    {
-        my $a       = shift
-        or return;
-
-        my $b       = shift
-        or return;
-
-        List::Util::first
-        {
-            $a->[$_] cmp $b->[$_]
-        }
-        ( 0 .. $#$a )
-    };
-
-    my $listh   = shift;
-
-    $listh->sort( $compare )
+    wantarray
+    ?  @valz
+    : \@valz
 }
 
 ########################################################################
@@ -1073,8 +584,7 @@ sub splice
 
     # short circut if there is nothing to do.
 
-    $count > 0 || @_
-    or return;
+    $count > 0 || @_ or return;
 
     my $node    = $$listh
     or confess "Bogus splice: empty list handler";
@@ -1103,7 +613,7 @@ sub splice
         $tail->[0]  = [];
     }
 
-    # at this point $dead is either false or
+    # at this point $dead is either false or 
     # a runt linked list lacking its terminating
     # node.
     #
@@ -1115,7 +625,7 @@ sub splice
        $node    = $node->[0] = [ $node->[0], $_ ];
     }
 
-    # nothing to return or clean up if there
+    # nothing to return or clean up if there 
     # wasn't anything removed.
 
     $dead or return;
@@ -1128,7 +638,7 @@ sub splice
     if( defined wantarray )
     {
         # hand back a linked list with $dead as the head node.
-
+        
         my $new     = $listh->new;
 
         my $head    = $$new;
@@ -1139,10 +649,9 @@ sub splice
     }
     else
     {
-        # explicitly wipe the remainder by 
-        # zeroing out the next ref.
+        $cleanup->( $dead );
 
-        @$dead  = ();
+        return
     }
 }
 
@@ -1265,109 +774,62 @@ LinkedList::Single - singly linked list manager.
     # using arrays, which are a helluva lot more
     # effective for the purpose).
 
+
     $list->push( @node_data );
 
     my @node_data   = $list->shift; # array of values
     my $node_data   = $list->shift; # arrayref
 
-    # shift and unshift manipulate the $listh root node 
-    # directly. This leaves $listh in the same location,
-    # which may be at the end of a list for more efficient
-    # handling of a FIFO queue.
+    # these manipulate the head node directly
+    # and to not modify the current list.
 
     $listh->unshift( @new_head_node_data );
 
     my @data    = $listh->shift;
 
-    # unshift + relocate.
-
-    $listh->unshift( @new_head_node_data )->head;
-
     # sequences of pushes are efficient for adding
-    # longer lists. this leaves $queue at the tail: 
+    # longer lists.
 
-    my $queue   = LinkedList::Single->new;
-    $queue->push( @$_ )
-    for @preloaded_data;
+    my $wcurve  = LinkedList::Single->new;
 
-    # in this case $queue itself stays at the end,
-    # with next_out entries coming from the root.
+    while( my $base = substr $dna, 0, 1, '' )
+    {
+        ...
 
-    my $queue   = LinkedList::Single->new;
-    
-    sub next_in  { $queue->push( @_ ) };
-    sub next_out { $queue->shift      };
-
-    # as are unshifts. a LIFO queue does not require 
-    # updating $listh:
-
-    $listh->unshift( @new_item );
-    $listh->shift( @next_item );
+        $wcurve->push( $r, $a, ++$z );
+    }
 
     # reset to the start-of-list.
 
-    $listh->head;
+    $wcurve->head;
 
     # hide extra data in the head node.
 
-    $listh->set_meta( @whatever_you_like );
+    $wcurve->set_meta( $z, $species );
 
     # extra data can come back as a list
     # or arrayref.
 
-    my ( $value )   = $wcurve->get_meta;
-    my $stuff       = $wcurve->get_meta;
+    my ( $size )    = $wcurve->get_meta;
 
-    # walk down the list examining a value from
-    # each node.
+
+    # walk down the list examining each item until
+    # the tail is reached.
     #
-    # this assumes that none of the internal
-    # nodes on the list are empty. For cases
-    # where that might happen see below.
-    #
-    # various ways to handle the empty return:
+    # unlike Perl's each there is no internal
+    # mechanism for re-setting the current node,
+    # if you don't call head $listh->each returns
+    # immediatley.
 
-    for( $listh->head ;;)
+    $listh->head;
+
+    while( my @data = $listh->each )
     {
-        # examine one value without the
-        # overhead of an array.
-
-        my ( $value ) = $listh->each;
-        $value // last;
-        ...
-    }
-
-    for( $listh->head ;;)
-    {
-        my @data    = $listh->each or last;
-        my $value   = shift @data;
-        ...
-    }
-
-    for( $listh->head ;; )
-    {
-        my $data    = $listh->each or last;
-        my $value   = $data->[0];
-        ...
-    }
-
-    # dealing with possibly empty internal nodes.
-    # (see also t/04-each.t).
-
-    for( $listh->head ;; )
-    {
-        $listh->has_next
-        or last;
-
-        # skip empty nodes.
-
-        my @data = $listh->each 
-        or next;
+        # play with the data
     }
 
     # duplicate a list handler, reset it to the
-    # start-of-list. note that clone produces a
-    # new list handler, not an entirely new list.
+    # start-of-list.
 
     if( $some_test )
     {
@@ -1382,16 +844,15 @@ LinkedList::Single - singly linked list manager.
         ...
     }
 
+
     # for those do-it-yourselfers in the crowd:
 
-    my $node    = $listh->head_node;
+    my $node    = $listh->root_node;
 
     while( @$node )
     {
-        # advance the node and extract the data.
-        # the final step will leave $node on the
-        # sentinel, which has no next pointer,
-        # which leaves @$node false above.
+        # advance the node and extract the data
+        # in one step.
 
         ( $node, @data )    = @$node;
 
@@ -1403,6 +864,29 @@ LinkedList::Single - singly linked list manager.
 
     my $head_node   = $listh->root->splice( 1 );
 
+    # if you prefer OO...
+    # $listh->each returns each value in order then
+    # returns false. one catch: in a list context
+    # this will return equally false for an empty
+    # node as the end-of-list.
+    #
+    # in a scalar context each returns false for the
+    # end-of-list and an empty arrayref for a node.
+
+    $listh->head;
+
+    while( my $data = $listh->each )
+    {
+        # deal with @$data
+    }
+
+    # if you *know* the nodes are never empty
+
+    while( my @data = $listh->each )
+    {
+        # deal with @data
+    }
+
     # note that $listh->next->node_data may be empty
     # even if there are mode nodes due to a node
     # having no data.
@@ -1413,6 +897,7 @@ LinkedList::Single - singly linked list manager.
 
     # $listh->head->cut is slower version of shift.
 
+
 =head1 DESCRIPTION
 
 Singly-linked list managed via ref-to-scalar.
@@ -1463,27 +948,6 @@ the existing node's state has to be kept.
     # note that $listh is unaffected by the
     # head or walking via each.
 
-Note that this leaves two root nodes referencing the head
-node, with no way to determine which one to weaken. At
-that point cleaning up the clone is necessary for garbage
-collecting the list.
-
-=item clone_list
-
-This clones the entire list or a subset of it.
-
-If "tail" has a true value then the list will be cloned with
-the current node as the head; if count is provided then tail
-is assumed to be true and up to count nodes will be copied
-(or up to the tail in any case).
-
-    my $whole_clone = $listh->clone_list;
-
-Clones are relatively quick and can handle large subets of data
-if the nodes' data are small lists or references. This makes a
-reasonable way to analyze a subset of the list in a loop.
-
-
 =item set_meta add_meta get_meta
 
 These allow storing list-wide data in the head.
@@ -1497,21 +961,11 @@ data for overloading.
 
 has_nodes is true if the list has any nodes
 at all; has_next is true if the current node
-has a next link; is_empty is true if the
-current node has no data.
+has a next link.
 
-Empty nodes can happen if the node contents are
-directly manipulated or a new node is added with
-no data in the first place.
-
-The boolean operator returns true if the node 
-has a next pointer, which will be true for all
-nodes other than the tail.
-
-The tail node is_empty it's also false. If nodes
-may contain empty data then using the boolean 
-or has_next are a better way to check for the 
-end of a list:
+The boolean overload is true if the current
+node has a next link (i.e., if calling $listh->next
+will go anywhere).
 
     sub walk_the_list
     {
@@ -1524,8 +978,7 @@ end of a list:
 
         while( $listh )
         {
-            my @data    = $list->node_data;
-            ...
+            # deal with the data in this node
         }
         continue
         {
@@ -1533,18 +986,6 @@ end of a list:
         }
     }
 
-or just 
-
-    for
-    (
-            $listh->head
-        ;   $listh
-        :   $listh->next
-    )
-    {
-        ...
-    }
-
 =item data set_data clear_data
 
 These return or set the data. They cannot be combined
@@ -1582,8 +1023,8 @@ by functional code or derived classes.
 
 =item head_node root_node root
 
-The head node is the first one containing user
-data; the root node references the head and
+The head node is the first one containing user 
+data; the root node references the head and 
 contains any metadata stored with set_meta.
 
 head_node is useful for anyone that wants to walk the
@@ -1610,7 +1051,7 @@ the list's root node. Derived classes can override
 these methods to use another form of storage for
 the list root.
 
-For example, unshift has to insert a node before
+For example, unshift has to insert a node before 
 the current head. It uses $lish->root to get a
 root node and then:
 
@@ -1619,7 +1060,7 @@ root node and then:
 
 to create the new head node.
 
-If you want to splice off the head node then
+If you want to splice off the head node then 
 you need to start from the root node:
 
     $listh->root;
@@ -1631,7 +1072,7 @@ you need to start from the root node:
 head and next start the list at the top and walk the list.
 
 each is like Perl's each in that it returns data until
-the end-of-list is reached, at which point it returns
+the end-of-list is reached, at which point it returns 
 nothing. It makes no attempt, however, to initialize
 or reset the list, only walk it.
 
@@ -1668,113 +1109,6 @@ be sufficient:
         # play with @data
     }
 
-
-=item map
-
-    map( $handler               )
-    map( $handler, tail  => 1   )
-    map( $handler, count => N   )
-
-Similar to the map builtin, map on the linked list takes
-a coderef and returns a new list of results produced by
-passing each node's values to the handler. The optional
-second argument "tail" then the list is processed from
-the current node (vs. the head); Given "count" the list
-at most N nodes are returned.
-
-    # produce a new list with each node having data of
-    # the sum, min, max, average of values in each node
-    # of the current list.
-
-    my $summarize
-    = sub
-    {
-        # expanded content of each node is passed on the stack.
-
-        [
-            sum( @_ ),
-            min( @_ ),
-            max( @_ ),
-            avg( @_ )
-        ]
-    };
-
-    my $new_listh   = $listh->map( $summarize );
-
-    # each node's data is an arrayref of the corresponding
-    # input node in sorted order. the existing list has node
-    # data of an array ref.
-    #
-    # the new list is suitable for replacing the existing
-    # one with "new_root" for multi-pass processing.
-
-    my $sorter
-    = sub
-    {
-        my $data   = shift;
-
-        [
-            sort { $a <=> $b } @$data
-        ]
-    };
-
-    my $new_list    = $listh->map( $sorter );
-
-    $listh->new_root( $new_list );
-
-    # at this point the list has been replaced with a new one
-    # with each node's data in sorted order.
-
-    # setting the optional second argument to true
-    # processes from the current node to the end of
-    # the list.
-    #
-    # in this case, get the average of
-    # values in each node from the current node to
-    # the end of the list.
-
-    my $avg         = sub { avg @_ };
-
-    my @subset_avg  = $listh->map( $avg, 1 );
-
-=item reduce
-
-    reduce( $handler                )
-    reduce( $handler, tail  => 1    )
-    reduce( $handler, count => N    )
-
-Similar to List::Util::reduce, but with a linked list
-as input rather than a perly list. The handler is
-assigned the result of calling
-
-    $handler( $curr_val, @node_data )
-
-for each node on the list. Note that this requires calling
-the handler once with the first node and an undef $curr_val.
-
-    my $handler = sub { shift + sum @_ };
-
-    my $total   = $listh->reduce( $handler );
-
-or the smallest total value in any node data:
-
-    my $handler
-    = sub
-    {
-        my $curr    = shift;
-        my $total   = sum @_;
-
-        $curr < $total
-        ? $total
-        : $curr
-    };
-
-    my $max_total   = $listh->reduce( $handler );
-
-See "map", above, for use of "tail" and "count" parameters.
-
-
-
 =item unshift shift push
 
 Notice the lack of "pop": it is quite expensive to
@@ -1809,23 +1143,23 @@ at the end-of-list after each push:
 
 This works well becuase shift and unshift do
 not modify the list handle's current node
-(i.e., $$listh is not touched). This means
+(i.e., $$listh is not touched). This means 
 that each push leaves the list handle at the
 end-of-list node, where the push is cheap.
 
 If $listh is not already at the end-of-list
 then push gets expensive since the list has
 to be traversed in order to find the end and
-perform the push. If you need to walk the list
-to scan the contents between push and shift op's
-then clone the list handle and use one copy to
-walk the list and the other one to manage the
+perform the push. If you need to walk the list 
+to scan the contents between push and shift op's 
+then clone the list handle and use one copy to 
+walk the list and the other one to manage the 
 queue.
 
     my $jobz    = LinkedList::Single->new;
     my $queue   = $jobz->clone;
 
-    # use $queue->push, $queue->shift for
+    # use $queue->push, $queue->shift for 
     # managing the queue, $jobs->head and
     # $jobs->each to scan the list, say for
     # stale or high-priority jobs.
@@ -1857,7 +1191,7 @@ Each will leave the list handler on the last node:
             and next DATA;
         }
 
-        # at this point $listh is at the
+        # at this point $listh is at the 
         # end-of-list, where push is cheap.
 
         $listh->push( @$new );
@@ -1905,7 +1239,7 @@ but removes all nodes.
 
 =item replace
 
-There are times when it is useful to retain the
+There are times when it is useful to retain the 
 current list handler but replace the contents.
 
 This is equivalent to:
@@ -1921,7 +1255,7 @@ exist; the similarity is that any existing list is cleaned
 up.
 
 The main use of this is adding skip chains to an existing
-linked list. One example would be adding skip chains
+linked list. One example would be adding skip chains 
 for each letter of the alphabet in an alphabetically-
 sorted list:
 
@@ -1935,13 +1269,13 @@ sorted list:
 
     $skip_chainz{ $letter } = $skip;
 
-The advantage to this approach is being able to
+The advantage to this approach is being able to 
 maintain a single list and avoid traversing all
 of it in order to locate known starting points.
 
 A similar result can be had by storing $node and
 calling $listh->node( $node ) later on to reset
-the position. The downside here is that any
+the position. The downside here is that any 
 state in the list handler object is lost, where
 the separate lists can be manipulated seprately
 with things like $listh->head.
@@ -1954,9 +1288,9 @@ Steven Lembark <lembark@wrkhors.com>
 
 =head1 COPYRIGHT
 
-Copyright (C) 2009-2021 Steven Lembark.
+Copyright (C) 2009-2011 Steven Lembark.
 
 =head1 LICENSE
 
-This code can be used under the same terms as v5.28 or
+This code can be used under the same terms as v5.14 or
 any later version of Perl.
